/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

[scriptable, uuid(a5761968-6e1a-4f2d-8191-ec749602b178)]
interface nsISensitiveInfoHiddenURI : nsISupports
{
  /**
   * Returns the spec attribute with sensitive information hidden. This will
   * only affect uri with password. The password part of uri will be
   * transformed into "****".
   */
   AUTF8String getSensitiveInfoHiddenSpec();
};
