#define MOZ_SOURCE_STAMP 55124c7343021091a61fdddac9f22628ca09642a
#define MOZ_SOURCE_REPO https://hg.mozilla.org/releases/mozilla-beta
#define MOZ_SOURCE_URL https://hg.mozilla.org/releases/mozilla-beta/rev/55124c7343021091a61fdddac9f22628ca09642a
