/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "domstubs.idl"

interface mozIDOMWindowProxy;

/**
 * The nsIDOMWindowCollection interface is an interface for a
 * collection of DOM window objects.
 */

[uuid(8d64f457-fb8c-49ea-a359-cef30eed9774)]
interface nsIDOMWindowCollection : nsISupports
{
  /**
   * Accessor for the number of windows in this collection.
   */
  readonly attribute unsigned long    length;

  /**
   * Method for accessing an item in this collection by index.
   */
  mozIDOMWindowProxy        item(in unsigned long index);

  /**
   * Method for accessing an item in this collection by window name.
   */
  mozIDOMWindowProxy        namedItem(in DOMString name);
};
